(***************************************************************************)
(***************************************************************************)
(****                                                                  	****)
(****		Copyright (c) 1995-1996 M. Maher AL-Rijleh											****)
(****                                                                   ****)
(****		Component 	: TNumberEdit	(32 Bit)															****)
(****		version		: 1.00																								****)
(****		Date			: 09/28/1995																					****)
(****		Last Update	: 05/01/1996																				****)
(**** 	Description	: An edit box that handle numerical entries					****)
(****                                                                  	****)
(***************************************************************************)
(***************************************************************************)

(***************************************************************************)
(***************************************************************************)
(****                                                                  	****)
(**** This code may be altered by you as you wish to make TNumberEdit		****)
(**** function to your personal needs. However, you cannot resell or 		****)
(***	redistribute the TNumberEdit as your own, altered or unaltered in ****)
(***	component or other non-executable form.														****)
(****                                                                  	****)
(***************************************************************************)
(***************************************************************************)

unit Dbnumedt;

interface

uses
  SysUtils, Classes, Controls, Forms, Messages, WinProcs, WinTypes, Graphics,
  Dialogs, NumEdit, DB, DBCtrls, DBTables, Buttons, W_Calc;

type

TDBNumberEdit = class(TNumberEdit)
  private
    { Private declarations }
    FDataLink: TFieldDataLink;
    Activate:Boolean;
    DataSetChangeing : Boolean;
    FAllowEscape: Boolean;
    FOldValue:TFloat;
    FCanvas: TControlCanvas;
    FAtExit:Boolean;
    FOldReadOnlyValue : Boolean;
    FNeedOnEnter      : Boolean;
    function GetDataField: string;
    function GetDataSource: TDataSource;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
  protected
    { Protected declarations }
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure DoExit; override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure DataChange(Sender: TObject);
    procedure EditingChange(Sender: TObject);
    procedure DoEnter; override;
  public
    { Public declarations }
    Constructor create(AOwner : TComponent); override;
    Destructor Destroy; Override;
  published
    { Published declarations }
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    Property AllowEscape: Boolean read FAllowEscape write FAllowEscape;
    property Alignment;
    property ActOnEnter;
    property BeepOnError;
    property NumericFormat;
    property DecimalFigures;
    property CurrencySign;
    property ThousandSeparator;
    property Decimalseparator;
    Property RoundNumber;
    Property ParenthesisForMinus;
    Property PositiveColor;
    Property NegativeColor;
    Property Value;
    Property MinimumAllowedValue;
    Property MaximumAllowedValue;
    property OutOfRangeErrorMessage;
  end;

{TDBCNumberEdit}
  TDBCNumberEdit = class(TDBNumberEdit)
  private
    { Private declarations}
    FButton : TBitBtn;
    FCaption : string;
    FPosition : TCalcPosition;
    FLeft : integer;
    FTop : integer;
    function GetVisibleButton:Boolean;
    procedure SetVisibleButton(value : boolean);
    function GetEnableButton:Boolean;
    procedure SetEnableButton(value : boolean);
  protected
    { Protected declarations }
    procedure ProcessClick (Sender:TObject);
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateWnd; override;
    procedure WMSize(var Message: TWMSize); message WM_Size;
    procedure keypress(var key:char); override;
    procedure Change; override;
    procedure DoEnter; override;
    procedure SetEditRect;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    Destructor Destroy; Override;
  published
    { Published declarations }
    property Alignment;
    property ActOnEnter;
    property BeepOnError;
    property NumericFormat;
    property DecimalFigures;
    property CurrencySign;
    property ThousandSeparator;
    property Decimalseparator;
    Property RoundNumber;
    Property ParenthesisForMinus;
    Property PositiveColor;
    Property NegativeColor;
    Property Value;
    Property MinimumAllowedValue;
    Property MaximumAllowedValue;
    property OutOfRangeErrorMessage;
    property CalcCaption : string read FCaption write FCaption;
    property CalcLeft : integer read FLeft write FLeft;
    property CalcPosition : TCalcPosition read FPosition write FPosition;
    property CalcTop : integer read FTop write FTop;
    property VisibleButton : Boolean read GetVisibleButton
    			write SetVisibleButton;
    property EnableButton : Boolean read GetEnableButton
    			write SetEnableButton;
  end;


implementation

type
	characterset = set of char;

const
	almostzero : TFLoat = -1;

var
	Registered : Boolean;


{TDBNumberEdit}
Constructor TDBNumberEdit.create(AOwner : TComponent);
begin
	inherited Create(AOwner);
  Activate := false;
	DataSetChangeing := false;
  AllowEscape := false;
  FAtExit := false;
	FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnEditingChange := EditingChange;
  FOldValue := 0;
  FOldReadOnlyValue := readonly;
end;

Destructor TDBNumberEdit.Destroy;
begin
	FDataLink.Free;
  FDataLink := nil;
  FCanvas.Free;
	inherited Destroy;
end;

procedure TDBNumberEdit.Change;
begin
	if DataSetChangeing then
  	exit;
  if (FDataLink <> nil) and (activate) and (FDataLink.DataSource <> nil) then
  begin
    FDataLink.Modified;
    inherited change;
    if not FAtExit then
    begin
    	if not (FDataLink.Editing) then
    	begin
    		FOldValue := self.value;
      	FDataLink.Edit;
    	end;
      if self.value <> FDataLink.Field.AsFloat then
    	  FDataLink.Field.AsFloat := self.value;
    end;
  end;
end;

procedure TDBNumberEdit.DataChange(Sender: TObject);
begin
	if (FDataLink = nil) or (FDataLink.DataSource = nil) or (FDataLink.Field = nil) then
  	exit;
	if FDataLink.Editing then
  	exit;
  DataSetChangeing := true;
  try
    if FDataLink.Field <> nil then
    begin
    	if FDataLink.Field.isnull then
      	self.value := 0
      else
      	self.value := FDataLink.Field.AsFloat;
      Activate := true;
    end;
  finally
  	DataSetChangeing := false;
  end;
end;

procedure TDBNumberEdit.EditingChange(Sender: TObject);
begin
  if (FDataLink = nil) or (FDataLink.DataSource = nil) or (FDataLink.Field = nil) then
  	exit;
	if FDataLink.Field.isnull then
  	self.value := 0;
  if FDataLink.DataSet.state in [dsinsert, dsedit] then
  begin
      self.readonly := FOldReadOnlyValue;
      if (not self.readonly) and FNeedOnEnter then
        inherited DoEnter;
  end else
    self.readonly := true;
end;


function TDBNumberEdit.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TDBNumberEdit.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TDBNumberEdit.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TDBNumberEdit.SetDataSource(Value: TDataSource);
begin
  FDataLink.DataSource := Value;
end;

procedure TDBNumberEdit.KeyPress(var Key: Char);
begin
	if (FDataLink = nil) or (FDataLink.DataSource = nil) or
  	(FDataLink.Field = nil) then
  begin
  	key := #0;
  	exit;
  end;
	if (readonly) or (FDataLink.Field.Readonly) then
  	key := #0;
	if (key = chr(VK_ESCAPE)) and (AllowEscape) and (FDataLink.Editing) then
  begin
  	self.value := FOldValue;
    key := #0;
  end;
  inherited KeyPress(Key);
end;

procedure TDBNumberEdit.DoEnter;
begin
  FNeedOnEnter := true;
  FOldReadOnlyValue := self.ReadOnly;
  if self.FDataLink.DataSet <> nil then
  begin
    if (not (self.FDataLink.DataSet.state in [dsinsert, dsedit]))
      and(not self.FDataLink.DataSource.AutoEdit = true) then
    begin
      self.readonly := true;
    end else
    begin
      inherited DoEnter;
      FNeedOnEnter := false;
    end;
  end;
end;

procedure TDBNumberEdit.DoExit;
begin
	FAtExit := true;
  try
  	inherited DoExit;
  finally
  	FATExit := false;
    self.ReadOnly := FOldReadOnlyValue;
  end;
end;

procedure TDBNumberEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

procedure TDBNumberEdit.WMPaint(var Message: TWMPaint);
var
  R: TRect;
  DC: HDC;
  PS: TPaintStruct;
begin
  if (FDataLink <> nil) and (FDataLink.DataSource <> nil) and
  	(FDataLink.Field <> nil) and (not FDataLink.Field.isnull) then
  begin
    inherited;
    Exit;
  end;

  if FCanvas = nil then
  begin
    FCanvas := TControlCanvas.Create;
    FCanvas.Control := Self;
  end;
  DC := Message.DC;
  if DC = 0 then DC := BeginPaint(Handle, PS);
  FCanvas.Handle := DC;
  try
    FCanvas.Font := Font;
    with FCanvas do
    begin
      R := ClientRect;
      if (BorderStyle = bsSingle) then
      begin
        Brush.Color := clWindowFrame;
        FrameRect(R);
      end;
      Brush.Color := Color;
      FillRect(R);
    end;
  finally
    FCanvas.Handle := 0;
    if Message.DC = 0 then EndPaint(Handle, PS);
  end;
end;

{TDBCNumberEdit}
constructor TDBCNumberEdit.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
   self.AutoSize := true;
   FButton := TBitBtn.Create(self);
   FButton.Parent := self;
   FButton.Height := self.Height - 4;
   FButton.Top := 0;
   FButton.width := ButtonWidth - 2;
   FButton.Caption :='...';
   FButton.left := self.width - FButton.width;
   FButton.style := bsNew;
   FButton.TabStop := False;
   FButton.OnClick := ProcessClick;
   Calcleft := (screen.width div 2) - 90;
   CalcTop 	:= (screen.Height div 2) - 145;
end;

destructor TDBCNumberEdit.destroy;
begin
	FButton.free;
  inherited destroy;
end;

procedure TDBCNumberEdit.keypress(var key:char);
begin
  if self.parent = nil then
     exit;
  inherited keypress(key);
  SetEditRect;
end;

procedure TDBCNumberEdit.Change;
begin
  if self.parent = nil then
     exit;
  inherited Change;
  SetEditRect;
end;

procedure TDBCNumberEdit.DoEnter;
begin
  if self.parent = nil then
     exit;
  inherited DoEnter;
  SetEditRect;
end;


procedure TDBCNumberEdit.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or WS_CLIPCHILDREN;
end;

procedure TDBCNumberEdit.CreateWnd;
var
  Loc: TRect;
begin
  inherited CreateWnd;
  SetEditRect;
end;

function TDBCNumberEdit.GetVisibleButton:Boolean;
begin
	result := FButton.width <> 0;
end;

procedure TDBCNumberEdit.SetVisibleButton(value : boolean);
begin
	if value then
  	FButton.Width := ButtonWidth
  else
  	FButton.Width := 0;
  RecreateWnd;
end;

function TDBCNumberEdit.GetEnableButton:Boolean;
begin
	result := FButton.Enabled;
end;

procedure TDBCNumberEdit.SetEnableButton(value : boolean);
begin
	if value = FButton.Enabled then
  	exit;
  FButton.Enabled := value;
end;

procedure TDBCNumberEdit.SetEditRect;
var
  Loc: TRect;
begin
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
  Loc.Bottom := ClientHeight;
  Loc.Right := ClientWidth - FButton.Width - 4;
  Loc.Top := 0;
  Loc.Left := 0;
  SendMessage(Handle, EM_SETRECTNP, 0, LongInt(@Loc));
  SendMessage(Handle, EM_GETRECT, 0, LongInt(@Loc));
end;

procedure TDBCNumberEdit.WMSize(var Message: TWMSize);
var
  Loc: TRect;
  MinHeight: Integer;
begin
  inherited;
  if FButton <> nil then
  begin
    FButton.SetBounds (Width - FButton.Width-4, 0,
    			FButton.Width, Height-4);
    SetEditRect;
  end;
end;

procedure TDBCNumberEdit.ProcessClick (Sender : TObject);
var
  Calculator: TCalculator;
begin
	Calculator := TCalculator.create(Application);
  try
		Calculator.Caption := FCaption;
  	if CalcPosition <> cpCenter then
  	begin
    	Calculator.Left := self.CalcLeft;
      Calculator.Top := self.CalcTop;
    end;
		Calculator.NumberEdit1.value := self.value;
  	if Calculator.Showmodal = mrok then
			self.value := Calculator.NumberEdit1.value;
  finally
  	Calculator.free;
  	self.Setfocus;
  end;
end;

end.
